(** * Fresh Constants *)

Set Implicit Arguments.

Require Import Coq.Arith.Max.
Require Export sublist.
Require Import language_syntax.

(** Remember that [constants] play the role of [parameters].
   In particular, we will show that fresh constants are as good as
   fresh parameters. *)

(** [new] chooses a fresh one which does not belong to lists. *)

Definition new (l:list name) : name := S (fold_right max 0 l).

Lemma In_less : forall (n:name)(l:list name),
  In n l ->
  n < new l.
Proof.
  unfold new; induction l; simpl; intro H; [elim H | ].
  destruct H as [HL | HR]; subst; auto with arith.
  apply lt_le_trans with (m:= S (fold_right max 0 l)); eauto 2 with arith.
Qed.

Lemma new_fresh : forall l : list name, (new l) # l.
Proof.
  intros l H.
  elim (lt_irrefl (new l)).
  apply In_less; exact H.  
Qed.
